clear
close all
clc

% Parameters
fs              = 44100;
N               = 1024;
f_vec           = (0:N-1) * fs/N;
x               = zeros(1,N);
x(1)            = 1;
g               = -1/sqrt(2);
M               = 8;

% Comb filter
b               = zeros(1,M+1);    	% Numerator coefficients
b(length(b))    = 1;
a               = zeros(1,M+1);     % Denominator coefficients
a(1)            = 1;
a(length(a))    = -g;
h_c             = filter(b,a,x);    % Impulse response

% Allpass filter
d               = zeros(1,M+1);     % Numerator coefficients
d(1)            = -g;
d(length(d))    = 1;
c               = zeros(1,M+1);     % Denominator coefficients
c(1)            = 1;
c(length(c))    = -g; 
h_a             = filter(d,c,x);    % Impulse response


%% Moving g to feedforward path

b_new               = zeros(1,M+1);         % New Numerator coefficients
b_new(length(b))    = g;
h_c_new             = filter(b_new,a,x);    % Impulse response

figure;
subplot(2,1,1);
stem(0:length(h_c)-1,h_c);
xlabel('n'); ylabel('h_c(n)');
xlim([0 128]); ylim([-1 1]); grid on;
title('g in Feedback path');
subplot(2,1,2);
stem(0:length(h_c_new)-1,h_c_new);
xlabel('n'); ylabel('h_{c,new}(n)');
xlim([0 128]); ylim([-1 1]); grid on;
title('g in Feedforward path');

%% Plot filter characteristics

figure;
% Comb Filter
subplot(2,3,1);
stem(0:length(h_c)-1,h_c);
xlabel('n'); ylabel('h_c(n)');
xlim([0 64]); ylim([-1 1]); grid on;
title('Impulse responses');
subplot(2,3,2);
plot(f_vec,20*log10(abs(fft(h_c,N))));
xlabel('f in Hz'); ylabel('|H_c(f)|');
xlim([0 fs/2]); ylim([-10 10]); grid on;
title('Magnitude responses');
subplot(2,3,3);
zplane(b,a);
xlabel('Real Part'); ylabel('Imaginary Part');
xlim([-1.5 1.5]); ylim([-1.5 1.5]); grid on;
title('Pole/Zero-Plots');
% Allpass Filter
subplot(2,3,4);
stem(0:length(h_a)-1,h_a);
xlabel('n'); ylabel('h_a(n)');
xlim([0 64]); ylim([-1 1]); grid on;
subplot(2,3,5);
plot(f_vec,20*log10(abs(fft(h_a,N))));
xlabel('f in Hz'); ylabel('|H_a(f)|');
xlim([0 fs/2]); ylim([-10 10]); grid on;
subplot(2,3,6);
zplane(d,c);
xlabel('Real Part'); ylabel('Imaginary Part');
xlim([-1.5 1.5]); ylim([-1.5 1.5]); grid on;

%% Filtering of audio signal

[x_audio, fs_audio]     = audioread('aint.wav');
x_audio                 = x_audio(:,1)';
y_c                     = filter(b,a,x_audio); 
y_a                     = filter(d,c,x_audio);

% Playback of audio signals
% sound(x_audio,fs_audio)
% pause
% sound(y_c,fs_audio)
% pause
% sound(y_a,fs_audio)